##############################################################################
#
# Copyright (c) 2004 TINY SPRL. (http://tiny.be) All Rights Reserved.
#                    Fabien Pinckaers <fp@tiny.Be>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import gtk
import gobject
from xml.parsers import expat

import rpc
import gettext
import parse

import copy

import time

def now():
	return time.strftime('%Y-%m-%d %H:%M')

class tree(object):
	def __init__(self, xml, fields, model=None, triggers=[], parent=None, state="draft", sel_multi=False):
		self.widget = gtk.TreeView()
		self.widget.set_reorderable(True)

		if not sel_multi:
			selection_mode = 'single'
		else:
			selection_mode = gtk.SELECTION_MULTIPLE
		self.widget.get_selection().set_mode(selection_mode)

		self.model = model
		self.fields = fields
		self.sort_col = False
		self.ids = {}

		p = parse.parse(self.fields)
		p.parse(xml, self.widget)
		self.name = p.title
		self.fields_order=p.fields_order
		self.colors={}
		if p.colors:
			self.colors=dict(map(lambda x: x.split(':'), p.colors.split(';')))

		self.widget.set_expander_column(self.widget.get_column(1))
		self.widget.set_headers_visible(True)

		types=[ gobject.TYPE_STRING ]
		for x in self.fields_order:
			types.append( fields_list_type.get(fields[x]['type'], gobject.TYPE_STRING))
		types.append( gobject.TYPE_STRING )
		self.model = gtk.ListStore(*types)
		self.widget.set_model(self.model)

		for i in range(len(types)-1):
			col = self.widget.get_column(i)
			col.add_attribute(col.get_cell_renderers()[0],'foreground', len(types)-1)

		self.widget.get_column(0).set_visible(False)
		for i in range(len(types)-1):
			col = self.widget.get_column(i)
			col.set_sort_column_id(i)
			col.set_reorderable(True)
			col.set_clickable(True)
		self.widget.show_all()

	def _func_sel_get(self, *args):
		pos = int(args[0].get_value(args[2], 0))
		if pos in self.ids:
			args[3].append(self.ids[pos])

	def sel_ids_get(self):
		sel = self.widget.get_selection()
		ids = []
		sel.selected_foreach(self._func_sel_get, ids)
		return ids

	def sel_id_get(self):
		return self.ids.get(self.pos_get(), False)

	def pos_get(self):
		sel = self.widget.get_selection().get_selected()
		if sel:
			model, iter = sel
			if not iter:
				return None
			pos = model.get_value(iter,0)
			return int(pos)
		return None

	def _value_get(self):
		return []

	def _value_set(self, val):
		self.model.clear()
		i = 0
		tochange = {}
		if len(val):
			for f in val[0].keys():
				if self.fields.has_key(f):
					if self.fields[f]['type'] in ('many2one','one2one'):
						ids = [v.get(f,'') for v in val if type(v[f])==type(1)]
						if len(ids):
							values = dict(rpc.session.rpc_exec_auth('/object', 'execute', self.fields[f]['relation'], 'name_get', ids))
							tochange[f] = values
					elif self.fields[f]['type'] in ('one2many','many2many'):
						ids = []
						for v in val:
							#previously was v[f] only (no for ...)
							if v[f] and type(v[f][0]) == tuple:
								ids += [x[1] for x in v[f]]
								ids = filter(lambda x : x , ids)
							else:
								ids += v[f]
						if len(ids):
							values = dict(rpc.session.rpc_exec_auth('/object', 'execute', self.fields[f]['relation'], 'name_get', ids))
							tochange[f] = values
						
		for i,r in enumerate(val):
			self.ids[i] = r.get('id', False)
			vals = [i]
			for field in self.fields_order:
				res = r[field]
				if type(res) == list and res and type(res[0]) == tuple:
					res = [x[1] for x in res]
				if not res and self.fields[field]['type'] in ('char','selection'):
					res = ''
				if res:
					if self.fields[field]['type'] in ('many2one','one2one'):
						if type(res)==type(1):
							res = tochange[field].get(r[field],'')
						elif type(res)==type([]):
							res = res[1]
					elif self.fields[field]['type'] in ('one2many','many2many'):
						res = ','.join(map(lambda x: (tochange.has_key(field) and tochange[field].get(x, '')) or '',res))[:50]
				vals.append(res)

			color = 'black'
			r['current_date'] = now()
			for col in self.colors:
				if eval(self.colors[col],r):
					color = col
			vals.append(color)
			self.model.append(vals)

	value = property(_value_get, _value_set, None,
	  'The content of the widget or ValueError if not valid')



fields_list_type = {
	'checkbox': gobject.TYPE_BOOLEAN,
	'integer': gobject.TYPE_INT,
	'float': gobject.TYPE_FLOAT
}


