<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">

	<xsl:import href="../../custom/corporate_defaults.xsl"/>
	<xsl:import href="../../base/report/rml_template.xsl"/>
	
	<xsl:decimal-format name="MyFormat" NaN="  " zero-digit="0"/>
	
	<xsl:template match="/">
		<xsl:call-template name="rml"/>
	</xsl:template>

	<!-- stylesheet -->

	<xsl:template name="stylesheet">
		<blockTableStyle id="accounts">
			<blockAlignment value="RIGHT" start="-1,0" stop="-1,-1"/>
			<lineStyle kind="LINEABOVE" start="-1,-2" stop="-1,-2"/>
		</blockTableStyle>

		<paraStyle name="notes" fontName="Helvetica" fontSize="8" alignment="justify"/>
		<paraStyle name="conditions" fontName="Helvetica" fontSize="8" alignment="justify"/>
		<paraStyle name="name" fontName="Helvetica-Bold"/>
		<paraStyle name="title" fontName="Helvetica-Bold" fontSize="22" alignment="center"/>
	 
		<blockTableStyle id="total">
			 <blockValign value="TOP"/>
			 <blockAlignment value="RIGHT"/>
			 <lineStyle kind="LINEABOVE" colorName="black" start="-1,0" stop="-1,0"/>
			 <lineStyle kind="LINEABOVE" colorName="black" start="-1,-1" stop="-1,-1"/>
			 <lineStyle kind="LINEBELOW" colorName="black" start="-1,-1" stop="-1,-1"/>
			 <lineStyle kind="LINEBEFORE" colorName="black" start="-1,-1" stop="-1,-1"/>
			 <lineStyle kind="LINEAFTER" colorName="black" start="-1,-1" stop="-1,-1"/>
			 <blockBackground colorName="(0.85,0.85,0.85)" start="-1,-1" stop="-1,-1"/>
		</blockTableStyle>
		
		<blockTableStyle id="description">
			 <blockBackground colorName="grey" start="0,0" stop="-1,0"/>
			 <blockTextColor colorName="white" start="0,0" stop="-1,0"/>
			 <blockValign value="TOP"/>
			 <blockAlignment value="RIGHT" start="1,0" stop="-1,-1"/>
			 <lineStyle kind="LINEBELOW" colorName="black" start="0,0" stop="-1,0"/>
		</blockTableStyle>
	</xsl:template>

	<xsl:template name="story">
		<xsl:apply-templates select="lots-list"/>
	</xsl:template>
	
	<xsl:template match="lots-list">
		<xsl:for-each select="buyer">
			<nextFrame/>
			<setNextTemplate name="other_pages"/>
			<para style="title" t="1">Attestation d'achat</para>
			<spacer length="1cm" width="2mm"/>
			
			<para><xsl:text t="1">Date de la vente : </xsl:text><xsl:value-of select="../date"/></para>
			<para><xsl:text>Etude : </xsl:text><xsl:value-of select="../etude"/></para>
			<para><xsl:text>Acheteur : </xsl:text><xsl:value-of select="name"/></para>
		
			<spacer length="0.3cm" width="1mm"/>
			<para>Description des lots :</para>
			<spacer length="0.3cm" width="1mm"/>
			<blockTable colWidths="8.5cm,3cm,3cm,3cm" style="description">
			<tr>
			<td><xsl:text t="1">Description</xsl:text></td>
			<td><xsl:text t="1">Prix d'adjudication</xsl:text></td>	
			<td><xsl:text t="1">Frais</xsl:text></td>
			<td><xsl:text t="1">Total</xsl:text></td>
			</tr>
			<xsl:for-each select="lot">
				<tr>
				<td><xsl:value-of select="description"/></td>
				<td>
					<xsl:value-of select="format-number(adj_price, '#,##0.00', 'MyFormat')"/>
				</td>
				<td>
					<xsl:value-of select="format-number(fee, '#,##0.00', 'MyFormat')"/>
				</td>
				<td>
					<xsl:value-of select="format-number(fee+adj_price, '#,##0.00', 'MyFormat')"/>
				</td>
				</tr>
			</xsl:for-each>
			</blockTable>
			<blockTable colWidths="14.5cm,3cm" style="total">
		   	<tr>
		       		<td t="1">TOTAL:</td>
		        	<td>
					<xsl:value-of select="format-number(sum(lot/adj_price[.!=''])+ sum(lot/fee[.!='']), '#,##0.00', 'MyFormat')"/>
				</td>
		       	</tr>
		       	</blockTable>
	
		<spacer length="0.3cm" width="1mm"/>
		<para></para>
		<spacer length="0.3cm" width="1mm"/>
		
		<para><xsl:text t="1">Cette attestation ne peut en aucun cas remplacer l'extrait du P.V. de vente judicaire. Ce dernier ne peut etre delivre que par l'huissier de justice instrumental.</xsl:text></para> 
			<setNextTemplate name="first_page"/>
		</xsl:for-each>
	</xsl:template>

</xsl:stylesheet>
